@@ -22,6 +22,20 @@ static gboolean option_disable_unsafe = FALSE;
 static char *option_base_uri = NULL;
 static char **uris = NULL;
 
+static char *
+get_relative_uri (const char *rel)
+{
+	GFile *file;
+	char *uri;
+
+	file = g_file_new_for_commandline_arg (rel);
+	uri = g_file_get_uri (file);
+	g_object_unref (file);
+	g_assert (uri != NULL);
+
+	return uri;
+}
+
 static char *
 test_relative_real (const char *uri, const char *output)
 {
@@ -128,10 +142,22 @@ test_data_get_data (const char *uri, guint *len)
 	/* Open the file. */
 	stream = g_file_read (file, NULL, &error);
 	if (stream == NULL) {
-		g_test_message ("URI '%s' couldn't be opened in test_data_get_data: '%s'", uri, error->message);
-		g_error_free (error);
-		return NULL;
+		GFile *dir;
+
+		/* Try to open the relative path in the source dir */
+		g_object_unref (file);
+		dir = g_file_new_for_path (TEST_SRCDIR);
+		file = g_file_get_child (dir, uri);
+		g_object_unref (dir);
+		stream = g_file_read (file, NULL, NULL);
+		if (stream == NULL) {
+			g_object_unref (file);
+			g_test_message ("URI '%s' couldn't be opened in test_data_get_data: '%s'", uri, error->message);
+			g_error_free (error);
+			return NULL;
+		}
 	}
+	g_object_unref (file);
 
 	buffer = g_malloc (MIME_READ_CHUNK_SIZE);
 	bytes_read = g_input_stream_read (G_INPUT_STREAM (stream), buffer, MIME_READ_CHUNK_SIZE, NULL, &error);
@@ -166,14 +192,15 @@ test_parsability (void)
 		gboolean slow;
 	} const files[] = {
 		/* NOTE: For relative paths, don't add a protocol. */
-		{ "560051.xml", TRUE, FALSE },
+		{ TEST_SRCDIR "560051.xml", TRUE, FALSE },
 		{ "itms://ax.itunes.apple.com/WebObjects/MZStore.woa/wa/viewPodcast?id=271121520&ign-mscache=1", TRUE, TRUE },
 		{ "http://phobos.apple.com/WebObjects/MZStore.woa/wa/viewPodcast?id=271121520", TRUE, TRUE },
 		{ "file:///tmp/file_doesnt_exist.wmv", FALSE, FALSE },
 		{ "http://live.hujjat.org:7860/main", FALSE, TRUE },
 		{ "http://www.comedycentral.com/sitewide/media_player/videoswitcher.jhtml?showid=934&category=/shows/the_daily_show/videos/headlines&sec=videoId%3D36032%3BvideoFeatureId%3D%3BpoppedFrom%3D_shows_the_daily_show_index.jhtml%3BisIE%3Dfalse%3BisPC%3Dtrue%3Bpagename%3Dmedia_player%3Bzyg%3D%27%2Bif_nt_zyg%2B%27%3Bspan%3D%27%2Bif_nt_span%2B%27%3Bdemo%3D%27%2Bif_nt_demo%2B%27%3Bbps%3D%27%2Bif_nt_bandwidth%2B%27%3Bgateway%3Dshows%3Bsection_1%3Dthe_daily_show%3Bsection_2%3Dvideos%3Bsection_3%3Dheadlines%3Bzyg%3D%27%2Bif_nt_zyg%2B%27%3Bspan%3D%27%2Bif_nt_span%2B%27%3Bdemo%3D%27%2Bif_nt_demo%2B%27%3Bera%3D%27%2Bif_nt_era%2B%27%3Bbps%3D%27%2Bif_nt_bandwidth%2B%27%3Bfla%3D%27%2Bif_nt_Flash%2B%27&itemid=36032&clip=com/dailyshow/headlines/10156_headline.wmv&mswmext=.asx", TRUE, TRUE },
+		{ TEST_SRCDIR "HackerMedley", TRUE, FALSE }, /* From https://bugzilla.redhat.com/show_bug.cgi?id=582850 and http://feeds.feedburner.com/HackerMedley */
 		{ NULL,  FALSE, FALSE }
-	};	
+	};
 
 	/* Loop through the list, downloading the URIs and checking for parsability */
 	for (i = 0; files[i].uri != NULL; ++i) {
@@ -185,7 +212,7 @@ test_parsability (void)
 		if (files[i].slow && !g_test_slow ())
 			continue;
 
-		g_test_message ("Testing \"%s\"...", files[i].uri);
+		g_test_message ("Testing data parsing \"%s\"...", files[i].uri);
 
 		data = test_data_get_data (files[i].uri, &len);
 		if (files[i].parsable == TRUE)
@@ -208,11 +235,87 @@ test_parsability (void)
 		if (files[i].slow && !g_test_slow ())
 			continue;
 
-		g_test_message ("Testing \"%s\"...", files[i].uri);
+		g_test_message ("Testing filename parsing \"%s\"...", files[i].uri);
 		g_assert (totem_pl_parser_can_parse_from_filename (files[i].uri, TRUE) == files[i].parsable);
 	}
 }
 
+static void
+entry_parsed_cb (TotemPlParser *parser,
+		 const char *uri,
+		 GHashTable *metadata,
+		 char **ret)
+{
+	if (*ret == NULL)
+		*ret = g_strdup (uri);
+}
+
+static char *
+parser_test_get_parse_result (const char *uri)
+{
+	TotemPlParserResult retval;
+	char *ret = NULL;
+	TotemPlParser *pl = totem_pl_parser_new ();
+
+	g_object_set (pl, "recurse", !option_no_recurse,
+			  "debug", option_debug,
+			  "force", option_force,
+			  "disable-unsafe", option_disable_unsafe,
+			  NULL);
+	g_signal_connect (G_OBJECT (pl), "entry-parsed",
+			  G_CALLBACK (entry_parsed_cb), &ret);
+
+	retval = totem_pl_parser_parse_with_base (pl, uri, option_base_uri, FALSE);
+	g_test_message ("Got retval %d for uri '%s'", retval, uri);
+	g_object_unref (pl);
+
+	return ret;
+}
+
+static void
+playlist_started_order (TotemPlParser *parser,
+			const char *uri,
+			GHashTable *metadata,
+			gboolean *ret)
+{
+	*ret = TRUE;
+}
+
+static void
+entry_parsed_cb_order (TotemPlParser *parser,
+		       const char *uri,
+		       GHashTable *metadata,
+		       gboolean *ret)
+{
+	/* Check that the playlist started happened before the entry appeared */
+	g_assert (*ret != FALSE);
+}
+
+static gboolean
+parser_test_get_order_result (const char *uri)
+{
+	TotemPlParserResult retval;
+	gboolean pl_started;
+	TotemPlParser *pl = totem_pl_parser_new ();
+
+	g_object_set (pl, "recurse", !option_no_recurse,
+			  "debug", option_debug,
+			  "force", option_force,
+			  "disable-unsafe", option_disable_unsafe,
+			  NULL);
+	g_signal_connect (G_OBJECT (pl), "playlist-started",
+			  G_CALLBACK (playlist_started_order), &pl_started);
+	g_signal_connect (G_OBJECT (pl), "entry-parsed",
+			  G_CALLBACK (entry_parsed_cb_order), &pl_started);
+
+	pl_started = FALSE;
+	retval = totem_pl_parser_parse_with_base (pl, uri, option_base_uri, FALSE);
+	g_test_message ("Got retval %d for uri '%s'", retval, uri);
+	g_object_unref (pl);
+
+	return pl_started;
+}
+
 static TotemPlParserResult
 simple_parser_test (const char *uri)
 {
@@ -226,22 +329,43 @@ simple_parser_test (const char *uri)
 			  NULL);
 
 	retval = totem_pl_parser_parse_with_base (pl, uri, option_base_uri, FALSE);
+	g_test_message ("Got retval %d for uri '%s'", retval, uri);
 	g_object_unref (pl);
 
 	return retval;
 }
 
+static void
+test_parsing_rtsp_text_multi (void)
+{
+	char *uri;
+	g_test_bug ("602127");
+	uri = get_relative_uri (TEST_SRCDIR "602127.qtl");
+	g_assert_cmpstr (parser_test_get_parse_result (uri), ==, "rtsp://host.org/video.mp4");
+	g_free (uri);
+}
+
+static void
+test_parsing_rtsp_text (void)
+{
+	char *uri;
+	uri = get_relative_uri (TEST_SRCDIR "single-line.qtl");
+	g_assert_cmpstr (parser_test_get_parse_result (uri), ==, "rtsp://host.org/video.mp4");
+	g_free (uri);
+}
+
 static void
 test_parsing_hadess (void)
 {
-	g_assert (simple_parser_test ("file:///home/hadess/Movies") != TOTEM_PL_PARSER_RESULT_ERROR);
+	if (g_strcmp0 (g_get_user_name (), "hadess"))
+		g_assert (simple_parser_test ("file:///home/hadess/Movies") == TOTEM_PL_PARSER_RESULT_SUCCESS);
 }
 
 static void
 test_parsing_nonexistent_files (void)
 {
 	g_test_bug ("330120");
-	g_assert (simple_parser_test ("file:///tmp/file_doesnt_exist.wmv") == TOTEM_PL_PARSER_RESULT_ERROR);
+	g_assert (simple_parser_test ("file:///tmp/file_doesnt_exist.wmv") == TOTEM_PL_PARSER_RESULT_SUCCESS);
 }
 
 static void
@@ -258,25 +382,88 @@ test_parsing_broken_asx (void)
 	g_assert (result != TOTEM_PL_PARSER_RESULT_ERROR);
 }
 
+static void
+test_parsing_out_of_order_asx (void)
+{
+	char *uri;
+	gboolean result;
+
+	/* From http://82.135.234.195/pukas.wax */
+	uri = get_relative_uri (TEST_SRCDIR "pukas.wax");
+	result = parser_test_get_order_result (uri);
+	g_free (uri);
+	g_assert (result != FALSE);
+}
+
 static void
 test_parsing_404_error (void)
 {
 	g_test_bug ("158052");
-	g_assert (simple_parser_test ("http://live.hujjat.org:7860/main") == TOTEM_PL_PARSER_RESULT_ERROR);
+	g_assert (simple_parser_test ("http://live.hujjat.org:7860/main") == TOTEM_PL_PARSER_RESULT_UNHANDLED);
 }
 
 static void
 test_parsing_xml_head_comments (void)
 {
+	char *uri;
 	g_test_bug ("560051");
-	g_assert (simple_parser_test ("file://" TEST_FILE_DIR "560051.xml") == TOTEM_PL_PARSER_RESULT_SUCCESS);
+	uri = get_relative_uri (TEST_SRCDIR "560051.xml");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_SUCCESS);
+	g_free (uri);
 }
 
 static void
 test_parsing_xml_comment_whitespace (void)
 {
+	char *uri;
 	g_test_bug ("541405");
-	g_assert (simple_parser_test ("file://" TEST_FILE_DIR "541405.xml") == TOTEM_PL_PARSER_RESULT_SUCCESS);
+	uri = get_relative_uri (TEST_SRCDIR "541405.xml");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_SUCCESS);
+	g_free (uri);
+}
+
+static void
+test_parsing_live_streaming (void)
+{
+	char *uri;
+	g_test_bug ("594036");
+	/* File from http://tools.ietf.org/html/draft-pantos-http-live-streaming-02#section-7.1 */
+	uri = get_relative_uri (TEST_SRCDIR "live-streaming.m3u");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_UNHANDLED);
+	g_free (uri);
+}
+
+static void
+test_parsing_xml_mixed_cdata (void)
+{
+	char *uri;
+	g_test_bug ("585407");
+	/* File from http://www.davidco.com/podcast.php */
+	uri = get_relative_uri (TEST_SRCDIR "585407.rss");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_SUCCESS);
+	g_free (uri);
+}
+
+static void
+test_parsing_not_asx_playlist (void)
+{
+	char *uri;
+	g_test_bug ("610471");
+	/* File from https://bugzilla.gnome.org/show_bug.cgi?id=610471#c0 */
+	uri = get_relative_uri (TEST_SRCDIR "asf-with-asx-suffix.asx");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_SUCCESS);
+	g_free (uri);
+}
+
+static void
+test_parsing_not_really_php (void)
+{
+	char *uri;
+	g_test_bug ("590722");
+	/* File from http://startwars.org/dump/remote_xspf.php */
+	uri = get_relative_uri (TEST_SRCDIR "remote_xspf.php");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_SUCCESS);
+	g_free (uri);
 }
 
 #define MAX_DESCRIPTION_LEN 128
@@ -297,6 +484,7 @@ entry_metadata_foreach (const char *key, const char *value, gpointer data)
 	    			break;
 			}
 	    	g_message ("\t%s = '%s' (truncated)", key, tmp);
+		g_free (tmp);
 	    	return;
 	}
 	if (g_ascii_strcasecmp (key, TOTEM_PL_PARSER_FIELD_PUB_DATE) == 0) {
@@ -410,6 +598,8 @@ main (int argc, char *argv[])
 		{ NULL }
 	};
 
+	setlocale (LC_ALL, "");
+
 	g_type_init ();
 	g_test_init (&argc, &argv, NULL);
 	g_test_bug_base ("http://bugzilla.gnome.org/show_bug.cgi?id=");
@@ -434,8 +624,15 @@ main (int argc, char *argv[])
 		g_test_add_func ("/parser/parsing/nonexistent_files", test_parsing_nonexistent_files);
 		g_test_add_func ("/parser/parsing/broken_asx", test_parsing_broken_asx);
 		g_test_add_func ("/parser/parsing/404_error", test_parsing_404_error);
+		g_test_add_func ("/parser/parsing/out_of_order_asx", test_parsing_out_of_order_asx);
 		g_test_add_func ("/parser/parsing/xml_head_comments", test_parsing_xml_head_comments);
 		g_test_add_func ("/parser/parsing/xml_comment_whitespace", test_parsing_xml_comment_whitespace);
+		g_test_add_func ("/parser/parsing/multi_line_rtsptext", test_parsing_rtsp_text_multi);
+		g_test_add_func ("/parser/parsing/single_line_rtsptext", test_parsing_rtsp_text);
+		g_test_add_func ("/parser/parsing/live_streaming", test_parsing_live_streaming);
+		g_test_add_func ("/parser/parsing/xml_mixed_cdata", test_parsing_xml_mixed_cdata);
+		g_test_add_func ("/parser/parsing/not_asx_playlist", test_parsing_not_asx_playlist);
+		g_test_add_func ("/parser/parsing/not_really_php", test_parsing_not_really_php);
 
 		return g_test_run ();
 	}
