@@ -130,6 +130,27 @@ parse_rss_item (TotemPlParser *parser, xml_node_t *parent)
 			duration = node->data;
 		} else if (g_ascii_strcasecmp (node->name, "length") == 0) {
 			filesize = node->data;
+		} else if (g_ascii_strcasecmp (node->name, "media:content") == 0) {
+			const char *tmp;
+
+			tmp = xml_parser_get_property (node, "type");
+			if (tmp != NULL &&
+			    g_str_has_prefix (tmp, "audio/") == FALSE)
+				continue;
+
+			tmp = xml_parser_get_property (node, "url");
+			if (tmp != NULL)
+				uri = tmp;
+			else
+				continue;
+
+			tmp = xml_parser_get_property (node, "fileSize");
+			if (tmp != NULL)
+				filesize = tmp;
+
+			tmp = xml_parser_get_property (node, "duration");
+			if (tmp != NULL)
+				duration = tmp;
 		} else if (g_ascii_strcasecmp (node->name, "enclosure") == 0) {
 			const char *tmp;
 
@@ -214,6 +235,7 @@ parse_rss_items (TotemPlParser *parser, const char *uri, xml_node_t *parent)
 				 TOTEM_PL_PARSER_FIELD_PUB_DATE, pub_date,
 				 TOTEM_PL_PARSER_FIELD_COPYRIGHT, copyright,
 				 TOTEM_PL_PARSER_FIELD_IMAGE_URI, img,
+				 TOTEM_PL_PARSER_FIELD_CONTACT, contact,
 				 NULL);
 
 	for (node = parent->child; node != NULL; node = node->next) {
@@ -233,10 +255,11 @@ TotemPlParserResult
 totem_pl_parser_add_rss (TotemPlParser *parser,
 			 GFile *file,
 			 GFile *base_file,
+			 TotemPlParseData *parse_data,
 			 gpointer data)
 {
-#ifndef HAVE_CAMEL
-	WARN_NO_CAMEL;
+#ifndef HAVE_GMIME
+	WARN_NO_GMIME;
 #else
 	xml_node_t* doc, *channel;
 	char *contents;
@@ -245,16 +268,16 @@ totem_pl_parser_add_rss (TotemPlParser *parser,
 	if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
-	totem_pl_parser_cleanup_xml (contents);
-	xml_parser_init (contents, size, XML_PARSER_CASE_INSENSITIVE);
-	if (xml_parser_build_tree_with_options (&doc, XML_PARSER_RELAXED | XML_PARSER_MULTI_TEXT) < 0) {
+	doc = totem_pl_parser_parse_xml_relaxed (contents, size);
+	if (doc == NULL) {
 		g_free (contents);
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 	}
+
 	/* If the document has no name */
 	if (doc->name == NULL
 	    || (g_ascii_strcasecmp (doc->name , "rss") != 0
-	    	&& g_ascii_strcasecmp (doc->name , "rss\n") != 0)) {
+		&& g_ascii_strcasecmp (doc->name , "rss\n") != 0)) {
 		g_free (contents);
 		xml_parser_free_tree (doc);
 		return TOTEM_PL_PARSER_RESULT_ERROR;
@@ -277,7 +300,7 @@ totem_pl_parser_add_rss (TotemPlParser *parser,
 	xml_parser_free_tree (doc);
 
 	return TOTEM_PL_PARSER_RESULT_SUCCESS;
-#endif /* !HAVE_CAMEL */
+#endif /* !HAVE_GMIME */
 }
 
 /* http://www.apple.com/itunes/store/podcaststechspecs.html */
@@ -285,10 +308,11 @@ TotemPlParserResult
 totem_pl_parser_add_itpc (TotemPlParser *parser,
 			  GFile *file,
 			  GFile *base_file,
+			  TotemPlParseData *parse_data,
 			  gpointer data)
 {
-#ifndef HAVE_CAMEL
-	WARN_NO_CAMEL;
+#ifndef HAVE_GMIME
+	WARN_NO_GMIME;
 #else
 	TotemPlParserResult ret;
 	char *uri, *new_uri, *uri_scheme;
@@ -303,22 +327,23 @@ totem_pl_parser_add_itpc (TotemPlParser *parser,
 	new_file = g_file_new_for_uri (new_uri);
 	g_free (new_uri);
 
-	ret = totem_pl_parser_add_rss (parser, new_file, base_file, data);
+	ret = totem_pl_parser_add_rss (parser, new_file, base_file, parse_data, data);
 
 	g_object_unref (new_file);
 
 	return ret;
-#endif /* !HAVE_CAMEL */
+#endif /* !HAVE_GMIME */
 }
 
 TotemPlParserResult
 totem_pl_parser_add_zune (TotemPlParser *parser,
 			  GFile *file,
 			  GFile *base_file,
+			  TotemPlParseData *parse_data,
 			  gpointer data)
 {
-#ifndef HAVE_CAMEL
-	WARN_NO_CAMEL;
+#ifndef HAVE_GMIME
+	WARN_NO_GMIME;
 #else
 	TotemPlParserResult ret;
 	char *uri, *new_uri;
@@ -341,12 +366,12 @@ totem_pl_parser_add_zune (TotemPlParser *parser,
 	new_file = g_file_new_for_uri (new_uri);
 	g_free (uri);
 
-	ret = totem_pl_parser_add_rss (parser, new_file, base_file, data);
+	ret = totem_pl_parser_add_rss (parser, new_file, base_file, parse_data, data);
 
 	g_object_unref (new_file);
 
 	return ret;
-#endif /* !HAVE_CAMEL */
+#endif /* !HAVE_GMIME */
 }
 
 /* Atom docs:
@@ -356,11 +381,11 @@ totem_pl_parser_add_zune (TotemPlParser *parser,
 static TotemPlParserResult
 parse_atom_entry (TotemPlParser *parser, xml_node_t *parent)
 {
-	const char *title, *author, *img, *uri, *filesize;
+	const char *title, *author, *uri, *filesize;
 	const char *copyright, *pub_date, *description;
 	xml_node_t *node;
 
-	title = author = img = uri = filesize = NULL;
+	title = author = uri = filesize = NULL;
 	copyright = pub_date = description = NULL;
 
 	for (node = parent->child; node != NULL; node = node->next) {
@@ -371,8 +396,6 @@ parse_atom_entry (TotemPlParser *parser, xml_node_t *parent)
 			title = node->data;
 		} else if (g_ascii_strcasecmp (node->name, "author") == 0) {
 			//FIXME
-		} else if (g_ascii_strcasecmp (node->name, "logo") == 0) {
-			img = node->data;
 		} else if (g_ascii_strcasecmp (node->name, "link") == 0) {
 			const char *rel;
 
@@ -483,10 +506,11 @@ TotemPlParserResult
 totem_pl_parser_add_atom (TotemPlParser *parser,
 			  GFile *file,
 			  GFile *base_file,
+			  TotemPlParseData *parse_data,
 			  gpointer data)
 {
-#ifndef HAVE_CAMEL
-	WARN_NO_CAMEL;
+#ifndef HAVE_GMIME
+	WARN_NO_GMIME;
 #else
 	xml_node_t* doc;
 	char *contents, *uri;
@@ -495,11 +519,12 @@ totem_pl_parser_add_atom (TotemPlParser *parser,
 	if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
-	xml_parser_init (contents, size, XML_PARSER_CASE_INSENSITIVE);
-	if (xml_parser_build_tree_with_options (&doc, XML_PARSER_RELAXED | XML_PARSER_MULTI_TEXT) < 0) {
+	doc = totem_pl_parser_parse_xml_relaxed (contents, size);
+	if (doc == NULL) {
 		g_free (contents);
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 	}
+
 	/* If the document has no name */
 	if (doc->name == NULL
 	    || g_ascii_strcasecmp (doc->name , "feed") != 0) {
@@ -516,17 +541,18 @@ totem_pl_parser_add_atom (TotemPlParser *parser,
 	xml_parser_free_tree (doc);
 
 	return TOTEM_PL_PARSER_RESULT_SUCCESS;
-#endif /* !HAVE_CAMEL */
+#endif /* !HAVE_GMIME */
 }
 
 TotemPlParserResult
 totem_pl_parser_add_xml_feed (TotemPlParser *parser,
 			      GFile *file,
 			      GFile *base_file,
+			      TotemPlParseData *parse_data,
 			      gpointer data)
 {
-#ifndef HAVE_CAMEL
-	WARN_NO_CAMEL;
+#ifndef HAVE_GMIME
+	WARN_NO_GMIME;
 #else
 	guint len;
 
@@ -536,14 +562,14 @@ totem_pl_parser_add_xml_feed (TotemPlParser *parser,
 	len = strlen (data);
 
 	if (totem_pl_parser_is_rss (data, len) != FALSE)
-		return totem_pl_parser_add_rss (parser, file, base_file, data);
+		return totem_pl_parser_add_rss (parser, file, base_file, parse_data, data);
 	if (totem_pl_parser_is_atom (data, len) != FALSE)
-		return totem_pl_parser_add_atom (parser, file, base_file, data);
+		return totem_pl_parser_add_atom (parser, file, base_file, parse_data, data);
 	if (totem_pl_parser_is_opml (data, len) != FALSE)
-		return totem_pl_parser_add_opml (parser, file, base_file, data);
+		return totem_pl_parser_add_opml (parser, file, base_file, parse_data, data);
 
 	return TOTEM_PL_PARSER_RESULT_UNHANDLED;
-#endif /* !HAVE_CAMEL */
+#endif /* !HAVE_GMIME */
 }
 
 /* From libgsf's gsf-utils.h */
@@ -569,16 +595,16 @@ check_header (char *data, gsize len)
 	unsigned flags;
 
 	if (len < 2 + 1 + 1 + 6)
-		return -1;
+		return 0;
 
 	/* Check signature */
 	if (memcmp (data, signature, sizeof (signature)) != 0)
-		return -1;
+		return 0;
 
 	/* verify flags and compression type */
 	flags  = data[3];
 	if (data[2] != Z_DEFLATED || (flags & ~GZIP_HEADER_FLAGS) != 0)
-		return -1;
+		return 0;
 
 	/* Get the uncompressed size */
 	/* FIXME, but how?  The size read here is modulo 2^32.  */
@@ -594,8 +620,8 @@ decompress_gzip (char *data, gsize len)
 	z_stream  stream;
 
 	retlen = check_header (data, len);
-	if (retlen < 0)
-		return NULL;
+	if (retlen == 0)
+		return g_strdup (data);
 
 	stream.zalloc    = (alloc_func)0;
 	stream.zfree     = (free_func)0;
@@ -638,7 +664,8 @@ totem_pl_parser_parse_itms_doc (xml_node_t *item)
 		/* What we're looking for looks like:
 		 * <key>feedURL</key><string>URL</string> */
 		if (g_ascii_strcasecmp (item->name, "key") == 0
-		    && g_ascii_strcasecmp (item->data, "feedURL") == 0) {
+		    && g_ascii_strcasecmp (item->data, "feedURL") == 0
+		    && item->next != NULL) {
 			item = item->next;
 			if (g_ascii_strcasecmp (item->name, "string") == 0)
 				return item->data;
@@ -655,7 +682,7 @@ totem_pl_parser_parse_itms_doc (xml_node_t *item)
 }
 
 static GFile *
-totem_pl_parser_get_feed_uri (const char *data, gsize len)
+totem_pl_parser_get_feed_uri (char *data, gsize len)
 {
 	xml_node_t* doc;
 	const char *uri;
@@ -663,8 +690,8 @@ totem_pl_parser_get_feed_uri (const char *data, gsize len)
 
 	uri = NULL;
 
-	xml_parser_init (data, len, XML_PARSER_CASE_INSENSITIVE);
-	if (xml_parser_build_tree_with_options (&doc, XML_PARSER_RELAXED | XML_PARSER_MULTI_TEXT) < 0)
+	doc = totem_pl_parser_parse_xml_relaxed (data, len);
+	if (doc == NULL)
 		return NULL;
 
 	/* If the document has no name */
@@ -712,25 +739,30 @@ TotemPlParserResult
 totem_pl_parser_add_itms (TotemPlParser *parser,
 			  GFile *file,
 			  GFile *base_file,
+			  TotemPlParseData *parse_data,
 			  gpointer data)
 {
-#ifndef HAVE_CAMEL
-	WARN_NO_CAMEL;
+#ifndef HAVE_GMIME
+	WARN_NO_GMIME;
 #else
 	char *contents, *uncompressed, *itms_uri;
-	GFile *itms_file, *feed_uri;
+	GFile *itms_file, *feed_file;
 	TotemPlParserResult ret;
 	gsize size;
 
 	if (g_file_has_uri_scheme (file, "itms") == FALSE) {
 		/* Get the webpage */
-		if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE)
+		if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE) {
+			DEBUG(file, g_print ("Couldn't load contents for %s\n", uri));
 			return TOTEM_PL_PARSER_RESULT_ERROR;
+		}
 
 		uncompressed = decompress_gzip (contents, size);
 		g_free (contents);
-		if (uncompressed == NULL)
+		if (uncompressed == NULL) {
+			DEBUG(file, g_print ("Couldn't decompress %s\n", uri));
 			return TOTEM_PL_PARSER_RESULT_ERROR;
+		}
 
 		/* Look for the link to the itms on phobos */
 		itms_uri = totem_pl_parser_get_itms_uri (uncompressed);
@@ -745,6 +777,7 @@ totem_pl_parser_add_itms (TotemPlParser *parser,
 	g_free (itms_uri);
 
 	if (g_file_load_contents (itms_file, NULL, &contents, &size, NULL, NULL) == FALSE) {
+		DEBUG(itms_file, g_print ("Couldn't load contents for itms_file %s\n", uri));
 		g_object_unref (itms_file);
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 	}
@@ -752,19 +785,25 @@ totem_pl_parser_add_itms (TotemPlParser *parser,
 
 	uncompressed = decompress_gzip (contents, size);
 	g_free (contents);
-	if (uncompressed == NULL)
+	if (uncompressed == NULL) {
+		DEBUG(itms_file, g_print ("Couldn't decompress itms_file %s\n", uri));
 		return TOTEM_PL_PARSER_RESULT_ERROR;
+	}
 
 	/* And look in the file for the feedURL */
-	feed_uri = totem_pl_parser_get_feed_uri (uncompressed, strlen (uncompressed) + 1);
-	if (feed_uri == NULL)
+	feed_file = totem_pl_parser_get_feed_uri (uncompressed, strlen (uncompressed) + 1);
+	if (feed_file == NULL) {
+		g_free (uncompressed);
 		return TOTEM_PL_PARSER_RESULT_ERROR;
+	}
 
-	ret = totem_pl_parser_add_rss (parser, feed_uri, NULL, NULL);
-	g_object_unref (feed_uri);
+	DEBUG(feed_file, g_print ("Found feed URI: %s\n", uri));
+
+	ret = totem_pl_parser_add_rss (parser, feed_file, NULL, parse_data, NULL);
+	g_object_unref (feed_file);
 
 	return ret;
-#endif /* !HAVE_CAMEL */
+#endif /* !HAVE_GMIME */
 }
 
 gboolean
@@ -776,11 +815,14 @@ totem_pl_parser_is_itms_feed (GFile *file)
 
 	uri = g_file_get_uri (file);
 
-	if (g_file_has_uri_scheme (file, "itms") != FALSE
-	    && strstr (uri, "phobos.apple.com") != NULL
-	    && strstr (uri, "viewPodcast") != NULL) {
-	    	g_free (uri);
-		return TRUE;
+	if (g_file_has_uri_scheme (file, "itms") != FALSE) {
+		if (strstr (uri, "phobos.apple.com") != NULL ||
+		    strstr (uri, "itunes.apple.com") != NULL) {
+			if (strstr (uri, "viewPodcast") != NULL) {
+				g_free (uri);
+				return TRUE;
+			}
+		}
 	}
 
 	if (strstr (uri, "phobos.apple.com/") != NULL
@@ -803,10 +845,10 @@ static TotemPlParserResult
 parse_opml_outline (TotemPlParser *parser, xml_node_t *parent)
 {
 	xml_node_t* node;
-	const char *title, *uri;
 
-	title = uri = NULL;
 	for (node = parent->child; node != NULL; node = node->next) {
+		const char *title, *uri;
+
 		if (node->name == NULL || g_ascii_strcasecmp (node->name, "outline") != 0)
 			continue;
 
@@ -858,10 +900,11 @@ TotemPlParserResult
 totem_pl_parser_add_opml (TotemPlParser *parser,
 			  GFile *file,
 			  GFile *base_file,
+			  TotemPlParseData *parse_data,
 			  gpointer data)
 {
-#ifndef HAVE_CAMEL
-	WARN_NO_CAMEL;
+#ifndef HAVE_GMIME
+	WARN_NO_GMIME;
 #else
 	xml_node_t* doc;
 	char *contents, *uri;
@@ -870,11 +913,12 @@ totem_pl_parser_add_opml (TotemPlParser *parser,
 	if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
-	xml_parser_init (contents, size, XML_PARSER_CASE_INSENSITIVE);
-	if (xml_parser_build_tree_with_options (&doc, XML_PARSER_RELAXED | XML_PARSER_MULTI_TEXT) < 0) {
+	doc = totem_pl_parser_parse_xml_relaxed (contents, size);
+	if (doc == NULL) {
 		g_free (contents);
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 	}
+
 	/* If the document has no name */
 	if (doc->name == NULL
 	    || g_ascii_strcasecmp (doc->name , "opml") != 0) {
@@ -891,7 +935,7 @@ totem_pl_parser_add_opml (TotemPlParser *parser,
 	xml_parser_free_tree (doc);
 
 	return TOTEM_PL_PARSER_RESULT_SUCCESS;
-#endif /* !HAVE_CAMEL */
+#endif /* !HAVE_GMIME */
 }
 
 #endif /* !TOTEM_PL_PARSER_MINI */
